from pymasmovil.client import Client


class Contract:
    """
    Parent class to OrderItem and Asset,
    which share a very similar class structure and attributes
    """

    id = ""
    account_id = ""
    name = ""
    surname = ""
    productName = ""
    phone = ""
    status = ""
    createdDate = ""
    productId = ""
    productRelation = ""
    initDate = ""
    attributes = {
        "Apellidos": "",
        "ICCID_Donante": "",
        "Nombre": "",
        "Numero_de_Documento": "",
        "Operador_Donante_Movil": "",
        "Operador_Receptor_Movil": "",
        "Tipo_de_Documento": "",
        "Tipo_de_Linea": "",
        "Fecha_de_solicitud_del_abonado": "",
        "Porcentaje_Consumo_Bono": "",
    }
    additionalBonds = []
    promotions = []
    simAttributes = {
        "ICCID": "",
        "IMSI": "",
        "PIN": "",
        "PIN2": "",
        "PUK": "",
        "PUK2": "",
    }
    tarAttributes = {
        "Fecha_planificada_de_entrega": "",
    }

    def __init__(self, **kwargs):

        for key, value in kwargs.items():
            if key in ["attributes", "simAttributes", "tarAttributes"]:
                attribute_dict = getattr(self, key).copy()
                for inner_key, inner_value in value.items():
                    if inner_key in attribute_dict:
                        attribute_dict[inner_key] = inner_value
                setattr(self, key, attribute_dict)
            else:
                if hasattr(self.__class__, key):
                    setattr(self, key, value)

    @classmethod
    def get(cls, session, contract_id):
        """
        Returns a Contract instance (order_item or asset) obtained by id.
        :param contract_id:

        :return: Contract:
        """

        response = Client(session).get(route="{}/{}".format(cls._route, contract_id))

        return response



from pymasmovil.client import Client
from pymasmovil.models.order_item import OrderItem
from pymasmovil.models.asset import Asset
from pymasmovil.errors.exceptions import (AccountRequiredParamsError,
                                          OrderItemNotFoundByICC)


class Account:
    _v1_route = "/v1/accounts"
    _v2_route = "/v2/accounts"

    town = ""
    surname = ""
    stair = ""
    roadType = ""
    roadNumber = ""
    roadName = ""
    region = ""
    province = ""
    postalCode = ""
    phone = ""
    name = ""
    id = ""
    flat = ""
    email = ""
    door = ""
    documentType = ""
    documentNumber = ""
    corporateName = ""
    buildingPortal = ""

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if hasattr(Account, key):
                setattr(self, key, value)

    @classmethod
    def get(cls, session, account_id):
        """
        Returns an account instance obtained by id.

        :param id:
        :return: Account:
        """

        account_response = Client(session).get(
            route="{}/{}".format(cls._v1_route, account_id)
        )

        return cls(**account_response)

    @classmethod
    def get_by_NIF(cls, session, NIF):
        """
        Returns an account instance searched by its NIF number.

        :param NIF: Document ID
        :return: Account
        """

        params = {"rowsPerPage": 1, "actualPage": 1, "documentNumber": NIF}

        response = Client(session).get(route=cls._v2_route, **params)

        return cls(**response["rows"][0])

    @classmethod
    def create(cls, session, **new_account):
        """
        Creates an account instance.

        :param **new_account:
        :return:
        """

        cls._check_required_attributes(new_account)

        new_account_id = Client(session).post(cls._v2_route, (), new_account)

        return cls(id=new_account_id, **new_account)

    def get_asset_by_phone(self, session, phone):
        return Asset(**self._get_contract_by_phone("assets", session, phone))

    def get_order_item_by_phone(self, session, phone):
        return OrderItem(**self._get_contract_by_phone("order-items", session, phone))

    def get_order_item_by_ICC(self, session, ICC):
        """
        Retrieve the order_item associated to a given account with a given ICC,
        from the MM sytem.

        :param: session: MM login session instance,
                ICC: ICC number of the order-item we need
        :return: most recently created  OrderItem object with corresponding ICC
        """
        params = {"rowsPerPage": 30, "actualPage": 1}

        response = Client(session).get(
            "{}/{}/order-items".format(self._v1_route, self.id), **params
        )
        order_item_list = response["rows"]

        icc_order_items = [order_item for order_item in order_item_list
                           if order_item.get("simAttributes").get("ICCID") == str(ICC)]

        if len(icc_order_items) == 0:
            raise OrderItemNotFoundByICC(ICC, self.id)
        elif len(icc_order_items) == 1:
            order_item = icc_order_items[0]
        else:
            order_item = self._get_newest_order_item(icc_order_items)

        return OrderItem(**order_item)

    def _get_contract_by_phone(self, entity, session, phone):
        """
        Retrieve the order_item/asset associated to a given account and filter one
        corresponding to a given phone number, from the MM sytem.

        :param: entity: string representing which object we want (order-items or asset)
                session: MM login session instance,
                phone: Phone number of the contract we expect
        :return: dict containing the object from the MM response
        """
        params = {"rowsPerPage": 1, "actualPage": 1, "phone": phone}

        response = Client(session).get(
            "{}/{}/{}".format(self._v1_route, self.id, entity), **params
        )

        return response["rows"][0]

    def _check_required_attributes(new_account):
        required_attributes = [
            "name",
            "surname",
            "documentNumber",
            "documentType",
            "email",
            "phone",
            "postalCode",
            "province",
            "region",
            "roadName",
            "roadNumber",
            "roadType",
            "town",
        ]

        if new_account.get("documentType") == "1":
            # CIF. Organization client
            required_attributes.append("corporateName")

        account_filled_attributes = set(filter(new_account.get, new_account.keys()))

        missing_attributes = set(required_attributes) - account_filled_attributes

        if len(missing_attributes) != 0:
            raise AccountRequiredParamsError(sorted(missing_attributes))

    def _get_newest_order_item(self, order_items):
        """
        From a list of dict formatted order_items, gets the newest one according to their
        `createdDate`.
        """
        sorted_order_items = sorted(order_items,
                                    key=lambda order_item: order_item.get("createdDate"),
                                    reverse=True)
        return sorted_order_items[0]



from pymasmovil.client import Client
from pymasmovil.errors.exceptions import (
    MissingLoginCredentialsError,
    AutenthicationError,
)
import os


class Session:
    def __init__(self, session_id):
        self.session_id = session_id

    @classmethod
    def create(cls):

        route = "/v2/login-api"

        login_params = {"domain": os.getenv("MM_DOMAIN")}
        auth_data = {
            "username": os.getenv("MM_USER"),
            "password": os.getenv("MM_PASSWORD"),
        }

        if auth_data["username"] is None:
            raise MissingLoginCredentialsError("MM_USER")
        elif auth_data["password"] is None:
            raise MissingLoginCredentialsError("MM_PASSWORD")
        elif login_params["domain"] is None:
            raise MissingLoginCredentialsError("MM_DOMAIN")

        session = Client().post(route=route, params=login_params, body=auth_data)

        if not session["sessionId"]:
            raise AutenthicationError

        return Session(session["sessionId"])



from pymasmovil.client import Client
from pymasmovil.models.new_line_request import NewLineRequest
from pymasmovil.models.contract import Contract
from pymasmovil.errors.exceptions import NewLineRequestRequiredParamsError


class OrderItem(Contract):

    _route = "/v2/order-items"
    orderType = ""
    lastModifiedDate = ""

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def get(cls, session, order_item_id):
        contract = super().get(session, order_item_id)

        return cls(**contract["orderDetail"])

    @classmethod
    def create(cls, session, account, **new_order_item):
        """
        Creates an order-item and posts it to the given account

        :param account: account where we want to add the order-item
        :param **new_order_item:
        :return: order-item instance
        """

        post_route = "/v1/accounts/{}/order-items".format(account.id)

        cls._check_required_attributes(new_order_item)

        Client(session).post(post_route, (), new_order_item)

        new_line_request = NewLineRequest(new_order_item, account.id)

        return OrderItem(**new_line_request.to_order_item())

    def _check_required_attributes(new_order_item):
        """Check that all compulsary attributes for a portability request
        are present in the request dynamic fields"""

        required_attributes = ["idLine", "documentType", "iccid", "productInfo"]

        is_portability = False
        new_line_info = new_order_item["lineInfo"][0]

        if new_line_info.get("phoneNumber"):
            is_portability = True
            required_attributes.extend(["docid", "donorOperator"])

        if new_line_info.get("documentType") == "CIF":
            # Organization client
            required_attributes.append("corporateName")

        elif new_line_info.get("documentType") in ("NIF", "NIE", "Pasaporte"):
            # Particular client
            required_attributes.extend(["name", "surname"])

        order_item_filled_attributes = set(
            filter(new_line_info.get, new_line_info.keys())
        )

        missing_attributes = set(required_attributes) - order_item_filled_attributes

        if len(missing_attributes) != 0:
            raise NewLineRequestRequiredParamsError(
                is_portability, sorted(missing_attributes)
            )



from pymasmovil.client import Client
from pymasmovil.models.contract import Contract
from pymasmovil.errors.exceptions import UnknownMMError


class Asset(Contract):
    _route = "/v2/assets"

    maxNumTariff = ""
    numTariff = ""
    assetType = ""

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def get(cls, session, asset_id):
        contract = super().get(session, asset_id)

        return cls(**contract["asset"])

    @classmethod
    def get_by_phone(cls, session, phone):
        """
        Retrieve the asset associated to a given phone number from the MM sytem.

        :param:
            phone (str): Phone number of the contract we expect
        :return: dict containing the asset from the MM response
        """
        params = {"rowsPerPage": 1, "actualPage": 1, "phone": phone}

        response = Client(session).get(cls._route, **params)

        return cls(**response["rows"][0])

    @classmethod
    def update_product(
        cls,
        session,
        asset_id,
        transaction_id,
        product_id="",
        execute_date="",
        additional_bonds=[],
        bonds_to_remove=[],
        promotions=[],
        promotions_to_remove=[],
    ):
        """
        Modify asset request:
            - change tariff
            - add one-shot bonds

        :param:
            asset_id (str): MM Asset ID
            transaction_id (str): Unique and correlative 18-length numeric code
            productId (str): ID from the new tariff we want to apply
                            [only for change tariff]
            execute_date (str): request date [only for change tariff]
            additional_bonds (list): additional bonds to add
            bonds_to_remove (list): additional bonds to delete
            promotions (list): promotions to add
            bonds_to_remove (list): promotions to delete
        :return: modified asset
        """

        route = "{}/{}/change-asset".format(cls._route.replace("v2", "v1"), asset_id)

        active_change = {
            "transactionId": transaction_id,
            "assetInfo": {
                "executeDate": execute_date,
                "productId": product_id,
                "additionalBonds": additional_bonds,
                "removeBonds": [{"assetId": bond_id} for bond_id in bonds_to_remove],
                "promotions": promotions,
                "removePromotions": [{"assetId": bond_id} for bond_id
                                     in promotions_to_remove],
            }
        }

        response = Client(session).patch(route, (), active_change)

        if response == "Petición aceptada":
            return response
        else:
            raise UnknownMMError(response)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open("README.md") as f:
    README = f.read()


VERSION = "0.0.20"

setup(
    name="pymasmovil",
    version=VERSION,
    author="Coopdevs",
    author_email="gerard.funosas@somconnexio.coop",
    maintainer="Daniel Palomar, Pau Pérez, Gerard Funosas",
    url="https://gitlab.com/coopdevs/pymasmovil",
    description="Python client for the Más Móvil's B2B API",
    long_description=README,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=("tests", "docs")),
    include_package_data=True,
    zip_safe=False,
    install_requires=["requests"],
    test_suite="unittest2.collector",
    tests_require=["unittest2", "mock", "tox", "coverage", "httpretty"],
)
